<?php

namespace Drupal\Tests\violinist_projects\Kernel;

use Drupal\violinist_projects\UpdateResultValue;

/**
 * Test the config renderer.
 *
 * @group violinist_projects
 */
class LogAnalyzerTest extends ViolinistProjectsTestBase {

  /**
   * Test the analyzer.
   *
   * @dataProvider logAnalyzerProvider
   */
  public function testAnalyzer($output, $expected_status, $expected_human_readable, $expected_suggestions) {
    /** @var \Drupal\violinist_projects\LogAnalyzer $analyzer */
    $analyzer = $this->container->get('violinist_projects.log_analyzer');
    self::assertEquals($expected_status, $analyzer->getUpdateStatusFromOutput($output));
    self::assertEquals($expected_human_readable, $analyzer->getUpdateStatusHumanReadable($output));
    foreach ($expected_suggestions as $suggestion) {
      self::assertContains($suggestion, $analyzer->getUpdateSuggestionsFromPackageAndOutput('test', $output));
    }
  }

  /**
   * A dataprovider.
   */
  public static function logAnalyzerProvider() {
    return [
      [
        'Installing dependencies from lock file (including require-dev)
Verifying lock file contents can be installed on current platform.
Package operations: 9 installs, 0 updates, 0 removals

In PluginManager.php line 787:

  dealerdirect/phpcodesniffer-composer-installer contains a Composer plugin which is blocked by your allow-plugins config. You may add it to the list if you consider it safe.
  You can run "composer config --no-plugins allow-plugins.dealerdirect/phpcodesniffer-composer-installer [true|false]" to enable it (true) or disable it explicitly and suppress this exception (false)
  See https://getcomposer.org/allow-plugins


install [--prefer-source] [--prefer-dist] [--prefer-install PREFER-INSTALL] [--dry-run] [--download-only] [--dev] [--no-suggest] [--no-dev] [--no-autoloader] [--no-progress] [--no-install] [--audit] [--audit-format AUDIT-FORMAT] [-v|vv|vvv|--verbose] [-o|--optimize-autoloader] [-a|--classmap-authoritative] [--apcu-autoloader] [--apcu-autoloader-prefix APCU-AUTOLOADER-PREFIX] [--ignore-platform-req IGNORE-PLATFORM-REQ] [--ignore-platform-reqs] [--] [<packages>...]

',
        UpdateResultValue::NewPluginNotAllowed,
        'The update probably contains installing a plugin which was not there before, and it is not allowed by the config.',
        [
          'Do the update manually',
          'Push an update where the plugin is allowed either together with a manual update, or as a separate PR.',
        ],
      ],
      [
        '  - Applying patches for psr/log
    my-invalid-patch.patch (Hack trick)
   Could not apply patch! Skipping. The error was: Cannot apply patch my-invalid-patch.patch

Generating autoload files
5 packages you are using are looking for funding.
Use the `composer fund` command to find out more!
PHP CodeSniffer Config installed_paths set to ../../drupal/coder/coder_sniffer,../../sirbrillig/phpcs-variable-analysis,../../slevomat/coding-standard
No security vulnerability advisories found.
',
        UpdateResultValue::CannotApplyPatch,
        'The update failed because a patch included in your project could not be applied.',
        [
          'Check if the patch is still needed, or if it has been committed upstream.',
          'If the patch is still needed, check if the patch still applies.',
        ],
      ],
      [
        'Some random output',
        UpdateResultValue::Unknown,
        'Unknown reason',
        [],
      ],
    ];
  }

}
